package higa.jhcl.model;

import java.io.Serializable;

import higa.util.EventMulticaster;

public class BasicMatrixContainer implements MatrixModel,
  Serializable {

  private Object[][] items = new Object[0][0];
  private int rowCount;
  private int columnCount;
  private boolean eventEnabled = true;
  private EventMulticaster modelCaster =
    new EventMulticaster();

  public BasicMatrixContainer() {}

  public BasicMatrixContainer(int rowCount, int columnCount) {
    this.rowCount = rowCount;
    this.columnCount = columnCount;
    items = new Object[rowCount][columnCount];
  }

  public BasicMatrixContainer(Object[][] newItems) {
    _setItems(newItems);
  }

  public void setItems(Object[][] newItems) {
    _setItems(newItems);

    processModelEvent(new MatrixModelEvent(
      this, MatrixModelEvent.STRUCTURE_CHANGED));
  }

  private void _setItems(Object[][] newItems) {
    rowCount = newItems.length;
    columnCount = rowCount > 0 ? newItems[0].length : 0;
    items = newItems;
  }

  public Object[][] getItems() {
    Object[][] newItems = new Object[rowCount][columnCount];
    newItems = items;
    return newItems;
  }

  public void set(int rowIndex, int columnIndex, Object data) {
    if (canSet(rowIndex, columnIndex)) {
      items[rowIndex][columnIndex] = data;

      processModelEvent(new MatrixModelEvent(
        this, MatrixModelEvent.ITEM_CHANGED,
        new MatrixLocation(rowIndex, columnIndex)));
    }
  }

  public Object get(int rowIndex, int columnIndex) {
    if (!canSet(rowIndex, columnIndex)) {
      return null;
    }

    Object copy = new Object();
    copy = items[rowIndex][columnIndex];
    return copy;
  }

  public void setRowCount(int newRowCount) {
    if (rowCount == newRowCount) {
      return;
    }

    _setRowCount(newRowCount);

    processModelEvent(new MatrixModelEvent(
      this, MatrixModelEvent.STRUCTURE_CHANGED));
  }

  protected synchronized void _setRowCount(int newRowCount) {
    Object[][] newItems = new Object[newRowCount][columnCount];
    int min = Math.min(rowCount, newRowCount);
    for (int r = 0; r < min; r++) {
      System.arraycopy(items[r], 0,
      newItems[r], 0, items[r].length);
    }

    rowCount = newRowCount;
    items = newItems;
  }

  public int getRowCount() {
    return rowCount;
  }

  public void setColumnCount(int newColumnCount) {
    if (columnCount == newColumnCount) {
      return;
    }

    _setColumnCount(newColumnCount);

    processModelEvent(new MatrixModelEvent(
      this, MatrixModelEvent.STRUCTURE_CHANGED));
  }

  protected synchronized void _setColumnCount(
    int newColumnCount) {

    Object[][] newItems = new Object[rowCount][newColumnCount];
    int min = Math.min(columnCount, newColumnCount);
    for (int r = 0; r < rowCount; r++) {
      System.arraycopy(items[r], 0, newItems[r], 0, min);
    }

    columnCount = newColumnCount;
    items = newItems;
  }

  public int getColumnCount() {
    return columnCount;
  }

  public void addModelListener(
    MatrixModelListener listener) {

    modelCaster.add(listener);
  }

  public void removeModelListener(
    MatrixModelListener listener) {

    modelCaster.remove(listener);
  }

  public void setEventEnabled(boolean enable) {
    this.eventEnabled = enable;
  }

  public boolean isEventEnabled() {
    return eventEnabled;
  }

  public boolean canSet(int rowIndex, int columnIndex) {
    if (rowIndex >= 0 && rowIndex < rowCount &&
      columnIndex >= 0 && columnIndex < columnCount) {

      return true;
    } else {
      return false;
    }
  }

  public void addRow() {
    addRow(rowCount);
  }

  public void addRow(int rowIndex) {
    if (rowIndex < 0 || rowIndex > rowCount) {
      return;
    }

    _addRow(rowIndex);

    processModelEvent(new MatrixModelEvent(
      this, MatrixModelEvent.ROW_ADDED,
      new MatrixLocation(rowIndex, 0)));
  }

  protected synchronized void _addRow(int rowIndex) {
    Object[][] newItems =
      new Object[rowCount + 1][columnCount];

    if (rowIndex != 0) {
      for (int r = 0; r < rowIndex; r++) {
        System.arraycopy(items[r], 0,
          newItems[r], 0, columnCount);
      }
    }

    if (rowIndex != rowCount) {
      for (int r = rowIndex; r < rowCount; r++) {
        System.arraycopy(items[r], 0,
          newItems[r + 1], 0, columnCount);
      }
    }

    rowCount++;
    items = newItems;
  }

  public void removeRow(int rowIndex) {
    if (rowIndex < 0 || rowIndex >= rowCount) {
      return;
    }

    _removeRow(rowIndex);

    processModelEvent(new MatrixModelEvent(
      this, MatrixModelEvent.ROW_REMOVED,
      new MatrixLocation(rowIndex, 0)));
  }

  protected synchronized void _removeRow(int rowIndex) {
    rowCount--;
    Object[][] newItems =
      new Object[rowCount][columnCount];

    if (rowIndex != 0) {
      for (int r = 0; r < rowIndex; r++) {
        System.arraycopy(items[r], 0,
          newItems[r], 0, columnCount);
      }
    }

    if (rowIndex != rowCount) {
      for (int r = rowIndex; r < rowCount; r++) {
        System.arraycopy(items[r + 1], 0,
          newItems[r], 0, columnCount);
      }
    }

    items = newItems;
  }

  public void removeAllRows() {
    _removeAllRows();

    processModelEvent(new MatrixModelEvent(
      this, MatrixModelEvent.STRUCTURE_CHANGED));
  }

  protected synchronized void _removeAllRows() {
    Object[][] newItems = new Object[0][columnCount];
    items = newItems;
    rowCount = 0;
  }

  public void addColumn() {
    addColumn(columnCount);
  }

  public void addColumn(int columnIndex) {
    if (columnIndex < 0 || columnIndex > columnCount) {
      return;
    }

    _addColumn(columnIndex);

    processModelEvent(new MatrixModelEvent(
      this, MatrixModelEvent.COLUMN_ADDED,
      new MatrixLocation(0, columnIndex)));
  }

  protected synchronized void _addColumn(int columnIndex) {
    Object[][] newItems =
      new Object[rowCount][columnCount + 1];

    if (columnIndex == columnCount) {
      for (int r = 0; r < rowCount; r++) {
        System.arraycopy(items[r], 0,
          newItems[r], 0, columnCount);
      }
    } else if (columnIndex == 0) {
      for (int r = 0; r < rowCount; r++) {
        System.arraycopy(items[r], 0,
          newItems[r], 1, columnCount);
      }
    } else {
      for (int r = 0; r < rowCount; r++) {
        System.arraycopy(items[r], 0,
          newItems[r], 0, columnIndex);

        System.arraycopy(items[r], columnIndex,
          newItems[r], columnIndex + 1,
          columnCount - columnIndex);
      }
    }

    items = newItems;
    columnCount++;
  }

  public void removeColumn(int columnIndex) {
    if (columnIndex < 0 || columnIndex >= columnCount) {
      return;
    }

    _removeColumn(columnIndex);

    processModelEvent(new MatrixModelEvent(
      this, MatrixModelEvent.COLUMN_REMOVED,
      new MatrixLocation(0, columnIndex)));
  }

  protected synchronized void _removeColumn(int columnIndex) {
    columnCount--;
    Object[][] newItems = new Object[rowCount][columnCount];

    if (columnIndex == columnCount) {
      for (int r = 0; r < rowCount; r++) {
        System.arraycopy(items[r], 0,
          newItems[r], 0, columnCount);
      }
    } else if (columnIndex == 0) {
      for (int r = 0; r < rowCount; r++) {
        System.arraycopy(items[r], 1,
          newItems[r], 0, columnCount);
      }
    } else {
      for (int r = 0; r < rowCount; r++) {
        System.arraycopy(items[r], 0,
          newItems[r], 0, columnIndex);

        System.arraycopy(items[r], columnIndex + 1,
          newItems[r], columnIndex,
          columnCount - columnIndex);
      }
    }

    items = newItems;
  }

  public void removeAllColumns() {
    _removeAllColumns();

    processModelEvent(new MatrixModelEvent(
      this, MatrixModelEvent.STRUCTURE_CHANGED));
  }

  protected synchronized void _removeAllColumns() {
    Object[][] newItems = new Object[rowCount][0];
    items = newItems;
    columnCount = 0;
  }

  protected void processModelEvent(MatrixModelEvent e) {
    if (eventEnabled && modelCaster.hasListeners()) {
      modelCaster.dispatch(e);
    }
  }
}