package higa.jhcl.view;

import java.io.Serializable;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.io.IOException;

import java.awt.Insets;
import java.awt.Color;
import java.awt.SystemColor;
import java.awt.Graphics;
import java.awt.Dimension;
import java.awt.Rectangle;
import java.awt.Font;
import java.awt.FontMetrics;
import java.text.Format;

import higa.util.Alignment;
import higa.util.Formatter;

import higa.jhcl.model.ItemPainter;
import higa.jhcl.model.ItemPaintSite;

public class TextItemPainter
  implements ItemPainter, Serializable {

  private int alignment =
    (Alignment.LEFT | Alignment.MIDDLE);

  private Insets margins =
    new Insets(1,1,1,1);

  private transient Format formatter;

  public TextItemPainter() {}

  public TextItemPainter(int alignment) {
    this.alignment = alignment;
  }

  public TextItemPainter(Insets margins) {
    setMargins(margins);
  }

  public TextItemPainter(Format formatter) {
    this();
    setFormatter(formatter);
  }

  public TextItemPainter(int alignment,
    Insets margins) {

    this(alignment);
    setMargins(margins);
  }

  public TextItemPainter(int alignment,
    Insets margins, Format formatter) {

    this(alignment, margins);
    setFormatter(formatter);
  }

  public void setAlignment(int a) {
    if (!Alignment.valid(a)) {
      throw new IllegalArgumentException(
        "alignment:" + a);
    }

    alignment = a;
  }

  public int getAlignment() {
    return alignment;
  }

  public void setMargins(Insets margins) {
    this.margins = margins;
  }

  public Insets getMargins() {
    return margins;
  }

  public void setFormatter(Format formatter) {
    this.formatter = formatter;
  }

  public Format getFormatter() {
    return formatter;
  }

  public Dimension getPreferredSize(
    Object data, Graphics g,
    int state, ItemPaintSite site) {

    if (g == null) {
      return new Dimension();
    }

    Font of = g.getFont();
    Font f = site != null ?
      site.getFont() : of;

    if (f == null) {
      f = of;
    }

    if (f != null) {
      g.setFont(f);
    }

    FontMetrics fm = g.getFontMetrics(f);
    String text = Formatter.format(formatter, data);
    Insets m = site != null ?
      site.getItemMargins() : margins;

    if (m == null) {
      m = margins;
    }

    g.setFont(of);
    return new Dimension(
      m.left + m.right +
      fm.stringWidth(text),
      m.top + m.bottom + fm.getHeight());
  }

  public void paint(Object data, Graphics g, Rectangle r,
    int state, ItemPaintSite site) {

    Font of = g.getFont();
    Font f = site != null ?
      site.getFont() : of;

    if (f == null) {
      f = of;
    }

    g.setFont(f);

    Insets m = site != null ?
      site.getItemMargins() : margins;

    if (m == null) {
      m = margins;
    }

    int a = site != null ?
      site.getAlignment() : alignment;

    if (a == 0) {
      a = alignment;
    }

    Color oc = g.getColor();
    Color bg = site != null ?
      site.getBackground() : oc;

    if (bg == null) {
      bg = oc;
    }

    Color fg = site != null ?
      site.getForeground() : SystemColor.windowText;

    if (fg == null) {
      fg = SystemColor.windowText;
    }

    String text = Formatter.format(formatter, data);
    FontMetrics fm = g.getFontMetrics(f);

    int xOffset;
    switch (a & Alignment.HORIZONTAL) {
      default:
      case Alignment.LEFT:
        xOffset = m.left;
        break;
      case Alignment.CENTER:
        xOffset = (r.width - fm.stringWidth(text)) / 2;
        break;
      case Alignment.RIGHT:
        xOffset = r.width - fm.stringWidth(text) - m.right;
    }

    int yOffset;
    switch (a & Alignment.VERTICAL) {
      case Alignment.TOP:
        yOffset = m.top;
        break;
      default:
      case Alignment.MIDDLE:
        yOffset = (r.height - fm.getHeight()) / 2;
        break;
      case Alignment.BOTTOM:
        yOffset = r.height - m.bottom - fm.getHeight();
        break;
    }

    yOffset += fm.getLeading() + fm.getAscent();

    g.setColor(bg);
    g.fillRect(r.x, r.y, r.width, r.height);

    if (text != null) {
      if ((state & ItemPainter.DISABLED) != 0) {
        g.setColor(SystemColor.controlLtHighlight);
        g.drawString(text, r.x + xOffset + 1, r.y + yOffset + 1);
        g.setColor(SystemColor.controlShadow);
      } else {
        g.setColor(fg);
      }

      g.drawString(text, r.x + xOffset, r.y + yOffset);
    }

    g.setFont(of);
    g.setColor(oc);
  }

  private void writeObject(ObjectOutputStream s)
    throws IOException {

    s.defaultWriteObject();
    s.writeObject(
      formatter instanceof Serializable ? formatter : null);
  }

  private void readObject(ObjectInputStream s)
    throws IOException, ClassNotFoundException {

    s.defaultReadObject();
    Object data = s.readObject();
    if (data instanceof Format) {
      formatter = (Format)data;
    }
  }
}